#include "ofApp.h"
#include "FFT.h"
//--------------------------------------------------------------
void ofApp::setup(){
    soundStream.listDevices();
    soundStream.setup(this,  // self callback
                      0,     // no output
                      1,     // 1 input ( mono input )
                      44100, // sampling rate[Hz]
                      BUFSIZE, // buffer size
                      4);    // number of buffer size.
    
    gui.setup("ScratchInputZ");
    gui.add(threshold_power.set("Threshold power",0.2, 0.01, 2.0));
    gui.add(threshold_k.setup("Threshold Katamuki",1.99, 0.1, 5.0));
    
    mode = MODE_READY;
    time_mode_changed = ofGetElapsedTimeMillis();
    flg_pause = false;
    for( int i = 0; i < BUFSIZE; i++ ){
        sum_power[i] = 0.0;
    }
    
    font.load(ofToDataPath("font.ttf"), 72);
    alpha = 0;
}

//--------------------------------------------------------------
void ofApp::update(){

}

//--------------------------------------------------------------
void ofApp::draw(){
	ofClear(100);
	ofSetColor(255);
	ofDrawBitmapString(ofToString(ofGetFrameRate()), ofGetWidth()-100,10);
	ofDrawBitmapString("ScratchInputZ by Tetsuaki Baba, 2014", ofGetWidth()-300,ofGetHeight()-20);
	
  switch(mode)
  {
    case MODE_RECOGNIZER:
      ofSetColor(50,50,200);
      ofDrawBitmapString("Recognizer",ofGetWidth()-100,30);
      break;
    case MODE_LEARNER:
      ofSetColor(200,50,50);
      ofDrawBitmapString("Learner",ofGetWidth()-100,30);
      break;
    case MODE_READY:
      ofSetColor(50,200,50);
      ofDrawBitmapString("Ready",ofGetWidth()-100,30);
      break;
  }
  ofSetColor(255,255,255);
  
  // FFTÁµêÊûú„ÅÆÂπ≥Âùá„Éë„ÉØ„Éº„ÅÆ„É™„Ç¢„É´„Çø„Ç§„É†Ë°®Á§∫
  ofNoFill();
	ofSetLineWidth(1.0);
	ofBeginShape();
	for(int i = 0; i < average_power.size(); i++ ){
		ofVertex(i,
             ofGetHeight()/4 - 5*average_power[average_power.size()-1-i]);
    /*
    if( average_power[average_power.size()-1-i] > 1.0 ){
      ofDrawBitmapString(ofToString(average_power[average_power.size()-1-i]),
                         i,
                         ofGetHeight()/4 - 5*average_power[average_power.size()-1-i]);
    }
     */
    
	}
	ofEndShape();
  
  // Èü≥Â£∞Ê≥¢ÂΩ¢„ÅÆ„É™„Ç¢„É´„Çø„Ç§„É†Ë°®Á§∫
	ofNoFill();
	ofSetLineWidth(1.0);
	ofBeginShape();
	for(int i = 0; i < BUFSIZE; i++ ){
		ofVertex(ofGetWidth()*(i/(float)BUFSIZE),
						 ofGetHeight()/2 + sound[i]*100);
	}
	ofEndShape();
	
	ofBeginShape();
	for( int i = 0; i < BUFSIZE; i++ ){
    if( i*myfft.getFreqStep(44100, BUFSIZE) >= 3000 ){
      ofVertex(ofGetWidth()*(i/(float)BUFSIZE),
               ofGetHeight()-power[i]-100);
    }
    else{
      ofVertex(ofGetWidth()*(i/(float)BUFSIZE),
               ofGetHeight()-100);
    }
	}
	ofEndShape();

  
  // FFTÁµêÊûú„ÅÆ„É™„Ç¢„É´„Çø„Ç§„É†Ë°®Á§∫
  ofDrawBitmapString("freq[Hz]", 10, ofGetHeight()-100+20);
  for( int i = 0; i < BUFSIZE/2; i++ ){
    if( i%25 == 0 ){
      ofDrawBitmapString(ofToString(i*(int)myfft.getFreqStep(44100, BUFSIZE)), ofGetWidth()*(i/(float)BUFSIZE), ofGetHeight()-100+10);
      ofLine(ofGetWidth()*(i/(float)BUFSIZE),
             ofGetHeight()-100+10,
             ofGetWidth()*(i/(float)BUFSIZE),
             ofGetHeight()-100-10);
    }
    
  }

  for( int i = 0; i < 4; i++ ){
    ofDrawBitmapString("feature["+ofToString(i)+"] = "+ofToString(feature[i]),
                       ofGetWidth()-300, 10 + 10*i);
  }
    
    ofBeginShape();
    for( int i = 0; i < buf_recognizer.size(); i++){
        ofVertex(i, 300 + buf_recognizer[i]);
    }
    ofEndShape();
    
    
    ofSetColor(255,255,255, alpha);
    font.drawString(str_result, ofGetWidth()/2, ofGetHeight()/2);
    if( alpha > 0 ){
        alpha--;
    }
    gui.draw();
}

//--------------------------------------------------------------
void ofApp::audioIn(float * input, int bufferSize, int nChannels){

    if( flg_pause == true ){
        return;
    }
    
    for (int i = 0; i < bufferSize; i++){
        sound[i] = input[i];
    }
    
    
    float avg_power = 0.0f;
    myfft.powerSpectrum(0,
                        (int)BUFSIZE/2,
                        sound,
                        BUFSIZE,
                        &magnitude[0],
                        &phase[0],
                        &power[0],
                        &avg_power);
    
    
    average_power.push_back(avg_power);
    if( average_power.size() > ofGetWidth() ){
        average_power.erase(average_power.begin());
    }
    
    if( mode == MODE_RECOGNIZER ){
        buf_recognizer.push_back(avg_power);
    }
    else{
        if( buf_recognizer.size() <= 10 ){
            buf_recognizer.push_back(avg_power);
        }
        else if( buf_recognizer.size() > 10 ){
            buf_recognizer.erase(buf_recognizer.begin());
        }
    }
    
    if((ofGetElapsedTimeMillis()-time_mode_changed) > 3000 &&
       avg_power > threshold_power && mode == MODE_READY ){
        mode = MODE_RECOGNIZER;
        time_mode_changed = ofGetElapsedTimeMillis();
    }
    
    
    if((ofGetElapsedTimeMillis()-time_mode_changed) > 3000 &&
       mode != MODE_READY &&  avg_power <= threshold_power ){
        if( mode == MODE_RECOGNIZER ){
            
            recognizer();
            for( int i = 0; i < BUFSIZE; i++ ){
                sum_power[i] = 0.0;
            }
        }
        mode = MODE_READY;
    }
    
    
    
  if( mode == MODE_RECOGNIZER ){
    for( int i = 0; i < BUFSIZE; i++ ){
      sum_power[i] = sum_power[i] + power[i];
    }
  }
  
    
}

int ofApp::recognizer()
{
    float sum1,sum2,sum3;
    sum1 = sum2 = sum3 = 0.0;
    for( int i = 0; i < BUFSIZE; i++ ){
        float Hz = i*myfft.getFreqStep(44100, BUFSIZE);
        if( 0 < Hz && Hz < 3000 ){
            sum1 = sum1 + sum_power[i];
        }
        else if( 3000 <= Hz && Hz < 8000 ){
            sum2 = sum2 + sum_power[i];
        }
        else if( 10000 <= Hz && Hz < 20000 ){
            sum3 = sum3 + sum_power[i];
        }
    }
    feature[1] = sum1;
    feature[2] = sum2;
    feature[3] = sum3;
  
    int peak_count = 0;
    float d_avg_power[buf_recognizer.size()-1];
    for( int i = 1; i < buf_recognizer.size(); i++)
    {
        d_avg_power[i-1] = buf_recognizer[i]-buf_recognizer[i-1];
        if( d_avg_power[i-1] > threshold_k ){
            peak_count++;
            i=i+10;
        }
        
    }
    str_result = "Recognized Gesture";
    cout << "Peak Count: " << peak_count << endl;
    cout << "Result Recognizer" << endl;
  
    buf_recognizer.clear();
    alpha = 255;
  
}

void ofApp::keyPressed(int key){
  if( key == ' '){
    flg_pause = !flg_pause;
  }
}
void ofApp::keyReleased(int key)
{
}



